<?php
/**
 * @file
 * Contains default views on behalf of the transactions  module.
 */


function transactions_views_handlers() {
  $handlers= array(
    'info' => array(
      'path' => drupal_get_path('module', 'transactions') .'/views',
    ),
    'handlers' => array(
      'views_handler_field_transactions_payer_uid' => array(
        'parent'=> 'views_handler_field'
      ),
      'views_handler_field_transactions_payee_uid' => array(
       'parent'=> 'views_handler_field'
      ),
    ),
  );
  return $handlers;
}

function transactions_views_data() {
  $data = array();
  $data['cc_transactions']['table']['group']  = t('Transactions');
  $data['cc_transactions']['table']['base'] = array(
    'field' => 'nid',
    'title' => t('Transaction'),
    'help' => t('Money flows between users.'),
  );
  $data['cc_transactions']['table']['join'] = array(
    'node' => array(
      'left_field' => 'nid',
      'field' => 'nid',
    ),
  );

  $data['cc_transactions']['payer_uid'] = array(
    'title' => t('Purchaser'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('The user id who spent currency in the transaction.'),
      'handler' => 'views_handler_field_transactions_payer_uid'
    ),
    'filter' => array(
      'help' => t('Show only transactions where the current user paid'),
      'handler' => 'views_handler_filter_user_current'
    ),
    'argument' => array(
      'help' => t('The user who gave currency in the transaction.'),
      'handler' => 'views_handler_argument'
    ),
    'relationship' => array(
      'help' => t('The user who spent currency in the transaction.'),
      'base' => 'users',
      'field' => 'payer_uid',
      'handler' => 'views_handler_relationship',
      'label' => t('Purchaser')
    ),
  );
  $data['cc_transactions']['payee_uid'] = array(
    'title' => t('Seller'),
    'field' => array(
      'help' => t('The user id who received currency in the transaction.'),
      'handler' => 'views_handler_field_transactions_payee_uid',
    ),
    'filter' => array(
      'help' => t('Show only transactions where the current user was paid'),
      'handler' => 'views_handler_filter_user_current',
      'label' => t('User is seller'),
    ),
    'argument' => array(
      'help' => t('The user who received currency in the transaction.'),
      'handler' => 'views_handler_argument'
    ),
    'relationship' => array(
      'help' => t('The user who received currency in the transaction.'),
      'base' => 'users',
      'field' => 'payee_uid',
      'handler' => 'views_handler_relationship',
      'label' => t('Seller'),
    ),
  );
  $data['cc_transactions']['starter_uid'] = array(
    'title' => t('Starter'),
    'field' => array(
      'help' => t('The user id who initiated the transaction.'),
      'handler' => 'views_handler_field',
    ),
    'filter' => array(
      'help' => t('Show only transactions started or not started by a given account'),
      'handler' => 'views_handler_filter_equality',
    ),
    'argument' => array(
      'help' => t('Show only transactions started by a given account'),
      'handler' => 'views_handler_argument',
    ),
    'relationship' => array(
      'help' => t('The account id who initiated the transaction.'),
      'base' => 'users',
      'field' => 'starter_uid',
      'handler' => 'views_handler_relationship',
      'label' => t('starter'),
    ),
  );
  $data['cc_transactions']['completer_uid'] = array(
    'title' => t('Completer'),
    'field' => array(
      'help' => t('The account id who completed the transaction.'),
      'handler' => 'views_handler_field',
    ),
    'filter' => array(
      'help' => t('Show only transactions completed or not completed by a given account'),
      'handler' => 'views_handler_filter_equality',
    ),
    'argument' => array(
      'help' => t('Show only transactions completed by a given account'),
      'handler' => 'views_handler_argument',
    ),
    'relationship' => array(
      'help' => t('The user id who initiated the transaction.'),
      'base' => 'users',
      'field' => 'completer_uid',
      'handler' => 'views_handler_relationship',
      'label' => t('completer'),
    ),
  );
  $data['cc_transactions']['quality'] = array(
    'title' => t('Rating'), 
    'field' => array(
      'help' => t('Quality of the transaction according to the payer'),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
  );
  $data['cc_transactions']['quantity'] = array(
    'title' => t('Amount'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('Number of units of currency transferred'),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help' => t('Transactions above or below a certain value'),
      'handler' => 'views_handler_filter_numeric'
    ),
  );
  $data['cc_transactions']['state'] = array(
    'title' => t('Transaction state'), 
    'field' => array(
      'help' => t('Whether the transaction is completed or pending'),
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'help' => t('Whether the transaction is completed or pending'),
      'handler' => 'views_handler_filter_boolean_operator',
    ),
    //'argument' => array(
    //  'help' => t('Whether the transaction is completed or pending'),
    //  'handler' => 'views_handler_argument_transaction_state', //extends
    //),
  );
//this belongs with the cc_currencies module, but breaks nothing if module ins't installed
  $data['cc_transactions']['cid'] = array(
    'title' => t('Currency'), 
    'field' => array(
      'help' => t('The currency that the transaction was conducted in'),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'argument' => array(
      'help' => t('The currency id'),
      'handler' => 'views_handler_argument'
    ),
  );
  
  $data['cc_balance_cache']['table']['group'] = 'Balances';
  $data['cc_balance_cache']['table']['base'] = array(
    'field' => 'uid',
    'title' => t('User Balance'),
    'help' => t('Balances of users.'),
  );
  $data['cc_balance_cache']['table']['join'] = array(
    'users' => array(
      'left_field' => 'uid',
      'field' => 'uid',
    ),
  );
  $data['cc_balance_cache']['uid']=array(
    'title' => t('User'),
    'argument' => array(
      'help' => t("The user id"),
      'handler' => 'views_handler_argument',
    )
  );
  $data['cc_balance_cache']['cid']=array(
    'title' => t('Currency'),
    'argument' => array(
      'help' => t("The currency id"),
      'handler' => 'views_handler_argument',
    ),
    'field' => array(
      'help' => t("The currency id"),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    )
  );
  $data['cc_balance_cache']['balance']=array(
    'title' => t('Balance'),
    'field' => array(
      'help' => t("The user's total cleared income minus total cleared expenditure"),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    )
  );
  $data['cc_balance_cache']['pending_difference']=array(
    'title' => t('Pending Difference'),
    'field' => array(
      'help' => t("sum of all pending transactions, (@type)", array('@type'=>'')),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    )
  );
  $data['cc_balance_cache']['pending_balance']=array(
    'title' => t('Pending Balance'),
    'field' => array(
      'help' => t('balance including pending transactions, (@type)', array('@type'=>'')),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    )
  );
  $data['cc_balance_cache']['gross_income']=array(
    'title' => t('Gross Income'),
    'field' => array(
      'help' => t("The user's total cleared income ever"),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    )
  );
  $data['cc_balance_cache']['quality_mean']=array(
    'title' => t('Rating'),
    'field' => array(
      'help' => t("The user's average rating for this currency"),
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    )
  );
  return $data;
}


function transactions_views_default_views() {
  $view = cc_ins_and_outs_view();
  $views[$view->name] = $view;
  
  $view = cc_transaction_list_view();
  $views[$view->name] = $view;
  
  $view = transactions_pending_view();
  $views[$view->name] = $view;
  return $views;
}

function cc_ins_and_outs_view() {
  $view = new view;
  $view->name = 'cc_ins_and_outs';
  $view->description = 'show transactions from and to a user';
  $view->tag = 'transactions';
  $view->view_php = '';
  $view->base_table = 'node';
  $view->is_cacheable = FALSE;
  $view->api_version = 2;
  $view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
  $handler = $view->new_display('default', 'Defaults', 'default');
  $handler->override_option('fields', array(
    'created' => array(
      'label' => t('Date'),
      'date_format' => 'custom',
      'custom_date_format' => 'd/m/y',
      'exclude' => 0,
      'id' => 'created',
      'table' => 'node',
      'field' => 'created',
      'relationship' => 'none',
    ),
    'payee_uid' => array(
      'label' => t('Payee'),
      'link_to_seller' => 1,
      'exclude' => 0,
      'id' => 'payee_uid',
      'table' => 'cc_transactions',
      'field' => 'payee_uid',
      'relationship' => 'none',
    ),
    'title' => array(
      'label' => t('Description'),
      'link_to_node' => 0,
      'exclude' => 0,
      'id' => 'title',
      'table' => 'node',
      'field' => 'title',
      'relationship' => 'none',
    ),
    'quantity' => array(
      'label' => t('Amount'),
      'exclude' => 0,
      'id' => 'quantity',
      'table' => 'cc_transactions',
      'field' => 'quantity',
      'relationship' => 'none',
    ),
    'cid' => array(
      'label' => t('Currency'),
      'exclude' => 1,
      'id' => 'cid',
      'table' => 'cc_transactions',
      'field' => 'cid',
      'relationship' => 'none',
    ),
  ));
  $handler->override_option('sorts', array(
    'created' => array(
      'order' => 'DESC',
      'granularity' => 'minute',
      'id' => 'created',
      'table' => 'node',
      'field' => 'created',
      'relationship' => 'none',
    ),
  ));
  $handler->override_option('arguments', array(
    'payer_uid' => array(
      'default_action' => 'empty',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '',
      'default_argument_type' => 'fixed',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'not found',
      'id' => 'payer_uid',
      'table' => 'cc_transactions',
      'field' => 'payer_uid',
      'relationship' => 'none',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'transaction' => 0,
        'offer' => 0,
        'want' => 0,
        'page' => 0,
        'story' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(
        '1' => 0,
      ),
      'validate_argument_type' => 'tid',
      'validate_argument_php' => '',
    ),
    'created_year' => array(
      'default_action' => 'default',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '%2',
      'default_argument_type' => 'date',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'not found',
      'id' => 'created_year',
      'table' => 'node',
      'field' => 'created_year',
      'relationship' => 'none',
      'default_options_div_prefix' => '',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'offer' => 0,
        'want' => 0,
        'transaction' => 0,
        'page' => 0,
        'story' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(),
      'validate_argument_type' => 'tid',
      'validate_argument_php' => '',
    ),
    'created_month' => array(
      'default_action' => 'default',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '%3, %2',
      'default_argument_type' => 'date',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'not found',
      'id' => 'created_month',
      'table' => 'node',
      'field' => 'created_month',
      'relationship' => 'none',
      'default_options_div_prefix' => '',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'offer' => 0,
        'want' => 0,
        'transaction' => 0,
        'page' => 0,
        'story' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(),
      'validate_argument_type' => 'tid',
      'validate_argument_php' => '',
    ),
  ));
  $handler->override_option('filters', array(
    'type' => array(
      'operator' => 'in',
      'value' => array(
        'transaction' => 'transaction',
      ),
      'group' => '0',
      'exposed' => FALSE,
      'expose' => array(
        'operator' => FALSE,
        'label' => '',
      ),
      'id' => 'type',
      'table' => 'node',
      'field' => 'type',
      'relationship' => 'none',
    ),
  ));
  $handler->override_option('access', array(
    'type' => 'none',
  ));
  $handler->override_option('title', 'payments');
  $handler->override_option('footer_format', '1');
  $handler->override_option('footer_empty', 0);
  $handler->override_option('empty', t('There were no payments in this period'));
  $handler->override_option('empty_format', '1');
  $handler->override_option('items_per_page', 0);
  $handler->override_option('use_pager', '0');
  $handler->override_option('style_plugin', 'table');
  $handler = $view->new_display('attachment', 'incoming', 'attachment_1');
  $handler->override_option('fields', array(
    'created' => array(
      'label' => t('Date'),
      'date_format' => 'custom',
      'custom_date_format' => 'd/m/y',
      'exclude' => 0,
      'id' => 'created',
      'table' => 'node',
      'field' => 'created',
      'override' => array(
        'button' => 'Use default',
      ),
      'relationship' => 'none',
    ),
    'name_1' => array(
      'label' => t('Payer'),
      'link_to_user' => 1,
      'exclude' => 0,
      'id' => 'name_1',
      'table' => 'users',
      'field' => 'name',
      'relationship' => 'payer_uid',
    ),
    'title' => array(
      'label' => t('Description'),
      'link_to_node' => 0,
      'exclude' => 0,
      'id' => 'title',
      'table' => 'node',
      'field' => 'title',
      'relationship' => 'none',
      'override' => array(
        'button' => 'Use default',
      ),
    ),
    'quantity' => array(
      'label' => t('Amount'),
      'exclude' => 0,
      'id' => 'quantity',
      'table' => 'cc_transactions',
      'field' => 'quantity',
      'relationship' => 'none',
    ),
    'cid' => array(
      'label' => t('Currency'),
      'exclude' => 1,
      'id' => 'cid',
      'table' => 'cc_transactions',
      'field' => 'cid',
      'relationship' => 'none',
    ),
  ));
  $handler->override_option('arguments', array(
    'payee_uid' => array(
      'default_action' => 'ignore',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '',
      'default_argument_type' => 'fixed',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'not found',
      'id' => 'payee_uid',
      'table' => 'cc_transactions',
      'field' => 'payee_uid',
      'relationship' => 'none',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'transaction' => 0,
        'offer' => 0,
        'want' => 0,
        'page' => 0,
        'story' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(
        '1' => 0,
      ),
      'validate_argument_type' => 'tid',
      'validate_argument_php' => '',
    ),
    'created_year' => array(
      'default_action' => 'default',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '%2',
      'default_argument_type' => 'date',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'not found',
      'id' => 'created_year',
      'table' => 'node',
      'field' => 'created_year',
      'override' => array(
        'button' => 'Use default',
      ),
      'relationship' => 'none',
      'default_options_div_prefix' => '',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'offer' => 0,
        'want' => 0,
        'transaction' => 0,
        'page' => 0,
        'story' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(),
      'validate_argument_type' => 'tid',
      'validate_argument_php' => '',
    ),
    'created_month' => array(
      'default_action' => 'default',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '%3, %2',
      'default_argument_type' => 'date',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'not found',
      'id' => 'created_month',
      'table' => 'node',
      'field' => 'created_month',
      'override' => array(
        'button' => 'Use default',
      ),
      'relationship' => 'none',
      'default_options_div_prefix' => '',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'offer' => 0,
        'want' => 0,
        'transaction' => 0,
        'page' => 0,
        'story' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(),
      'validate_argument_type' => 'tid',
      'validate_argument_php' => '',
    ),
  ));
  $handler->override_option('title', t('Receipts'));
  $handler->override_option('footer', '<?php print _statement_month_pager(); ?>');
  $handler->override_option('footer_format', '3');
  $handler->override_option('footer_empty', 1);
  $handler->override_option('empty', t('There were no receipts in this period'));
  $handler->override_option('attachment_position', 'after');
  $handler->override_option('inherit_arguments', TRUE);
  $handler->override_option('inherit_exposed_filters', 1);
  $handler->override_option('displays', array(
    'default' => 'default',
    'page_1' => 0,
  ));
  return $view;
}

function cc_transaction_list_view() {
  $view = new view;
  $view->name = 'cc_transaction_list';
  $view->description = 'transactions from all users';
  $view->tag = 'transactions';
  $view->view_php = '';
  $view->base_table = 'node';
  $view->is_cacheable = FALSE;
  $view->api_version = 2;
  $view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
  $handler = $view->new_display('default', 'Defaults', 'default');
  $handler->override_option('relationships', array(
    'payer_uid' => array(
      'label' => t('Purchaser'),
      'required' => 1,
      'id' => 'payer_uid',
      'table' => 'cc_transactions',
      'field' => 'payer_uid',
      'relationship' => 'none',
    ),
    'payee_uid' => array(
      'id' => 'payee_uid',
      'table' => 'cc_transactions',
      'field' => 'payee_uid',
    ),
  ));
  $handler->override_option('fields', array(
    'changed' => array(
      'label' => t('Date'),
      'date_format' => 'custom',
      'custom_date_format' => 'd-m-y',
      'exclude' => 0,
      'id' => 'changed',
      'table' => 'node',
      'field' => 'changed',
      'relationship' => 'none',
    ),
    'name_1' => array(
      'label' => t('Purchaser'),
      'link_to_user' => 1,
      'exclude' => 0,
      'id' => 'name_1',
      'table' => 'users',
      'field' => 'name',
      'relationship' => 'payer_uid',
    ),
    'name_2' => array(
      'label' => t('Seller'),
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'alt' => '',
        'prefix' => '',
        'suffix' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
      ),
      'link_to_user' => 1,
      'overwrite_anonymous' => 0,
      'anonymous_text' => '',
      'exclude' => 0,
      'id' => 'name_2',
      'table' => 'users',
      'field' => 'name',
      'relationship' => 'payee_uid',
    ),
    'title' => array(
      'label' => 'Trade',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'alt' => '',
        'prefix' => '',
        'suffix' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'strip_tags' => 0,
        'html' => 0,
      ),
      'link_to_node' => 1,
      'exclude' => 0,
      'id' => 'title',
      'table' => 'node',
      'field' => 'title',
      'relationship' => 'none',
      'override' => array(
        'button' => 'Override',
      ),
    ),
    'quantity' => array(
      'label' => t('Amount'),
      'exclude' => 0,
      'id' => 'quantity',
      'table' => 'cc_transactions',
      'field' => 'quantity',
      'relationship' => 'none',
    ),
     'cid' => array(
      'label' => 'Currency',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'alt' => '',
        'prefix' => '',
        'suffix' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
      ),
      'exclude' => 1,
      'id' => 'cid',
      'table' => 'cc_transactions',
      'field' => 'cid',
      'override' => array(
        'button' => 'Override',
      ),
    )
  ));
  
  $handler->override_option('sorts', array(
    'created' => array(
      'order' => 'DESC',
      'granularity' => 'hour',
      'id' => 'created',
      'table' => 'node',
      'field' => 'created',
      'relationship' => 'none',
    ),
  ));
  $handler->override_option('arguments', array(
    'cid' => array(
      'default_action' => 'ignore',
      'style_plugin' => 'default_summary',
      'style_options' => array(),
      'wildcard' => 'all',
      'wildcard_substitution' => 'All',
      'title' => '',
      'default_argument_type' => 'fixed',
      'default_argument' => '',
      'validate_type' => 'none',
      'validate_fail' => 'ignore',
      'id' => 'cid',
      'table' => 'cc_transactions',
      'field' => 'cid',
      'relationship' => 'none',
      'default_options_div_prefix' => '',
      'default_argument_user' => 0,
      'default_argument_fixed' => '',
      'default_argument_php' => '',
      'validate_argument_node_type' => array(
        'currency' => 0,
        'offer' => 0,
        'want' => 0,
        'transaction' => 0,
        'page' => 0,
        'story' => 0,
        'testfield' => 0,
      ),
      'validate_argument_node_access' => 0,
      'validate_argument_nid_type' => 'nid',
      'validate_argument_vocabulary' => array(),
      'validate_argument_type' => 'tid',
      'user_argument_type' => 'uid',
      'restrict_user_roles' => 0,
      'user_roles' => array(),
      'validate_argument_php' => '',
    ),
  ));
  $handler->override_option('filters', array(
  'type' => array(
    'operator' => 'in',
    'value' => array(
      'transaction' => 'transaction',
    ),
    'group' => '0',
    'exposed' => FALSE,
    'expose' => array(
      'operator' => FALSE,
      'label' => '',
    ),
    'id' => 'type',
    'table' => 'node',
    'field' => 'type',
    'relationship' => 'none',
  ),
  'starter_uid' => array(
    'operator' => '!=',
    'value' => '1',
    'group' => '0',
    'exposed' => FALSE,
    'expose' => array(
      'operator' => FALSE,
      'label' => '',
    ),
    'id' => 'starter_uid',
    'table' => 'cc_transactions',
    'field' => 'starter_uid',
    'relationship' => 'none',
  ),
  'completer_uid' => array(
    'operator' => '!=',
    'value' => '1',
    'group' => '0',
    'exposed' => FALSE,
    'expose' => array(
      'operator' => FALSE,
      'label' => '',
    ),
    'id' => 'completer_uid',
    'table' => 'cc_transactions',
    'field' => 'completer_uid',
    'relationship' => 'none',
  ),
  ));
  $handler->override_option('access', array(
    'type' => 'perm',
    'perm' => 'view all transactions',
  ));
  $handler->override_option('title', t('Transaction list'));
  $handler->override_option('footer_format', '1');
  $handler->override_option('empty', t('There are no transactions in the system yet'));
  $handler->override_option('empty_format', '1');
  $handler->override_option('items_per_page', 50);
  $handler->override_option('use_pager', 'mini');
  $handler->override_option('distinct', 0);
  $handler->override_option('style_plugin', 'table');
  $handler->override_option('style_options', array(
    'grouping' => '',
    'override' => 1,
    'sticky' => 0,
    'order' => 'asc',
    'columns' => array(
      'changed' => 'changed',
      'name_1' => 'name_1',
      'name_2' => 'name_2',
      'title' => 'title',
      'quantity' => 'quantity',
      'cid' => 'cid',
      'quality' => 'quality',
      'state' => 'state',
    ),
    'info' => array(
      'changed' => array(
        'sortable' => 1,
        'separator' => '',
      ),
      'name_1' => array(
        'sortable' => 1,
        'separator' => '',
      ),
      'name_2' => array(
        'sortable' => 1,
        'separator' => '',
      ),
      'title' => array(
        'sortable' => 0,
        'separator' => '',
      ),
      'quantity' => array(
        'separator' => '',
      ),
      'cid' => array(
        'separator' => '',
      ),
      'quality' => array(
        'separator' => '',
      ),
      'state' => array(
        'sortable' => 0,
        'separator' => '',
      ),
    ),
    'default' => '-1',
  ));
  $handler = $view->new_display('page', 'Page', 'page_1');
  $handler->override_option('path', 'transactions');
  $handler->override_option('menu', array(
    'type' => 'normal',
    'title' => t('All transactions'),
    'description' => t('all transactions on the system, with optional argument for currency'),
    'weight' => 20,
    'name' => 'navigation',
  ));
  $handler->override_option('tab_options', array(
    'type' => 'none',
    'title' => '',
    'description' => '',
    'weight' => 0,
  ));
  $handler = $view->new_display('block', 'Block', 'block_1');
  $handler->override_option('fields', array(
    'name_1' => array(
      'label' => '',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'alt' => '',
        'prefix' => '',
        'suffix' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
      ),
      'link_to_user' => 1,
      'overwrite_anonymous' => 0,
      'anonymous_text' => '',
      'exclude' => 0,
      'id' => 'name_1',
      'table' => 'users',
      'field' => 'name',
      'relationship' => 'payer_uid',
      'override' => array(
        'button' => 'Use default',
      ),
    ),
    'quantity' => array(
      'label' => '',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'alt' => '',
        'prefix' => '',
        'suffix' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
      ),
      'exclude' => 0,
      'id' => 'quantity',
      'table' => 'cc_transactions',
      'field' => 'quantity',
      'relationship' => 'none',
      'override' => array(
        'button' => 'Use default',
      ),
    ),
    'name_2' => array(
      'label' => '',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'alt' => '',
        'prefix' => '',
        'suffix' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
      ),
      'link_to_user' => 1,
      'overwrite_anonymous' => 0,
      'anonymous_text' => '',
      'exclude' => 0,
      'id' => 'name_2',
      'table' => 'users',
      'field' => 'name',
      'relationship' => 'payee_uid',
      'override' => array(
        'button' => 'Use default',
      ),
    ),
  ));
  $handler->override_option('items_per_page', 10);
  $handler->override_option('use_pager', '0');
  $handler->override_option('style_plugin', 'list');
  $handler->override_option('style_options', array(
    'grouping' => '',
    'type' => 'ul',
  ));
  $handler->override_option('row_options', array(
    'inline' => array(
      'name_1' => 'name_1',
      'quantity' => 'quantity',
      'name_2' => 'name_2',
    ),
    'separator' => '>',
  ));
  $handler->override_option('block_description', '');
  $handler->override_option('block_caching', -1);
  return $view;  
}

function transactions_pending_view() {
  $view = new view;
$view->name = 'cc_transactions_pending';
$view->description = t('incomplete transactions, per user');
$view->tag = '';
$view->view_php = '';
$view->base_table = 'node';
$view->is_cacheable = FALSE;
$view->api_version = 2;
$view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
$handler = $view->new_display('default', 'Defaults', 'default');
$handler->override_option('sorts', array(
  'created' => array(
    'order' => 'ASC',
    'granularity' => 'hour',
    'id' => 'created',
    'table' => 'node',
    'field' => 'created',
    'relationship' => 'none',
  ),
));
$handler->override_option('arguments', array(
  'completer_uid' => array(
    'default_action' => 'ignore',
    'style_plugin' => 'default_summary',
    'style_options' => array(),
    'wildcard' => 'all',
    'wildcard_substitution' => 'All',
    'title' => '',
    'default_argument_type' => 'current_user',
    'default_argument' => '',
    'validate_type' => 'user',
    'validate_fail' => 'not found',
    'id' => 'completer_uid',
    'table' => 'cc_transactions',
    'field' => 'completer_uid',
    'validate_user_argument_type' => 'uid',
    'validate_user_roles' => array(
      '2' => 0,
      '3' => 0,
    ),
    'relationship' => 'none',
    'default_options_div_prefix' => '',
    'default_argument_user' => 0,
    'default_argument_fixed' => '',
    'default_argument_php' => '',
    'validate_argument_node_type' => array(
      'offer' => 0,
      'want' => 0,
      'requack' => 0,
      'transaction' => 0,
      'page' => 0,
      'story' => 0,
    ),
    'validate_argument_node_access' => 0,
    'validate_argument_nid_type' => 'nid',
    'validate_argument_vocabulary' => array(
      '2' => 0,
    ),
    'validate_argument_type' => 'tid',
    'validate_argument_transform' => 0,
    'validate_user_restrict_roles' => 0,
    'validate_argument_php' => '',
  ),
));
$handler->override_option('filters', array(
  'state' => array(
    'operator' => '=',
    'value' => '1',
    'group' => '0',
    'exposed' => FALSE,
    'expose' => array(
      'operator' => FALSE,
      'label' => '',
    ),
    'id' => 'state',
    'table' => 'cc_transactions',
    'field' => 'state',
    'relationship' => 'none',
  ),
));
$handler->override_option('access', array(
  'type' => 'none',
));
$handler->override_option('empty', t('There are no transactions to complete'));
$handler->override_option('items_per_page', 0);
$handler->override_option('row_plugin', 'node');
$handler = $view->new_display('page', 'user_page', 'page_1');
$handler->override_option('arguments', array(
  'completer_uid' => array(
    'default_action' => 'not found',
    'style_plugin' => 'default_summary',
    'style_options' => array(),
    'wildcard' => 'all',
    'wildcard_substitution' => 'All',
    'title' => '',
    'default_argument_type' => 'current_user',
    'default_argument' => '',
    'validate_type' => 'user',
    'validate_fail' => 'not found',
    'id' => 'completer_uid',
    'table' => 'cc_transactions',
    'field' => 'completer_uid',
    'validate_user_argument_type' => 'uid',
    'validate_user_roles' => array(
      '2' => 0,
      '3' => 0,
    ),
    'relationship' => 'none',
    'default_options_div_prefix' => '',
    'default_argument_user' => 0,
    'default_argument_fixed' => '',
    'default_argument_php' => '',
    'validate_argument_node_type' => array(
      'offer' => 0,
      'want' => 0,
      'requack' => 0,
      'transaction' => 0,
      'page' => 0,
      'story' => 0,
    ),
    'validate_argument_node_access' => 0,
    'validate_argument_nid_type' => 'nid',
    'validate_argument_vocabulary' => array(
      '2' => 0,
    ),
    'validate_argument_type' => 'tid',
    'validate_argument_transform' => 0,
    'validate_user_restrict_roles' => 0,
    'validate_argument_php' => '',
    'override' => array(
      'button' => 'Use default',
    ),
  ),
));
$handler->override_option('header', '<h4>' . t('Transactions for this user to complete') . '</h4>');
$handler->override_option('header_format', '2');
$handler->override_option('header_empty', 1);
$handler->override_option('path', 'marketplace/pending');
$handler->override_option('menu', array(
  'type' => 'none',
  'title' => '',
  'description' => '',
  'weight' => '0',
  'name' => 'navigation',
));
$handler->override_option('tab_options', array(
  'type' => 'none',
  'title' => '',
  'description' => '',
  'weight' => 0,
));
$handler = $view->new_display('attachment', 'Attachment', 'attachment_1');
$handler->override_option('arguments', array(
  'starter_uid' => array(
    'default_action' => 'not found',
    'style_plugin' => 'default_summary',
    'style_options' => array(),
    'wildcard' => 'all',
    'wildcard_substitution' => 'All',
    'title' => '',
    'default_argument_type' => 'fixed',
    'default_argument' => '',
    'validate_type' => 'user',
    'validate_fail' => 'not found',
    'id' => 'starter_uid',
    'table' => 'cc_transactions',
    'field' => 'starter_uid',
    'validate_user_argument_type' => 'uid',
    'validate_user_roles' => array(
      '2' => 0,
      '3' => 0,
    ),
    'override' => array(
      'button' => 'Use default',
    ),
    'relationship' => 'none',
    'default_options_div_prefix' => '',
    'default_argument_user' => 0,
    'default_argument_fixed' => '',
    'default_argument_php' => '',
    'validate_argument_node_type' => array(
      'offer' => 0,
      'want' => 0,
      'requack' => 0,
      'transaction' => 0,
      'page' => 0,
      'story' => 0,
    ),
    'validate_argument_node_access' => 0,
    'validate_argument_nid_type' => 'nid',
    'validate_argument_vocabulary' => array(
      '2' => 0,
    ),
    'validate_argument_type' => 'tid',
    'validate_argument_transform' => 0,
    'validate_user_restrict_roles' => 0,
    'validate_argument_php' => '',
  ),
));
$handler->override_option('header', '<h4>' . t('Transactions started by this user') . '</h4>');
$handler->override_option('header_format', '2');
$handler->override_option('header_empty', 1);
$handler->override_option('attachment_position', 'after');
$handler->override_option('inherit_arguments', TRUE);
$handler->override_option('inherit_exposed_filters', FALSE);
$handler->override_option('displays', array(
  'page_1' => 'page_1',
  'default' => 0,
));
$handler = $view->new_display('page', 'All', 'page_2');
$handler->override_option('arguments', array());
$handler->override_option('path', 'marketplace/all_pending');
$handler->override_option('menu', array(
  'type' => 'none',
  'title' => '',
  'description' => '',
  'weight' => 0,
  'name' => 'navigation',
));
$handler->override_option('tab_options', array(
  'type' => 'none',
  'title' => '',
  'description' => '',
  'weight' => 0,
));
$handler = $view->new_display('block', 'my_pending_block', 'block_1');
$handler->override_option('arguments', array(
  'completer_uid' => array(
    'default_action' => 'default',
    'style_plugin' => 'default_summary',
    'style_options' => array(),
    'wildcard' => 'all',
    'wildcard_substitution' => 'All',
    'title' => '',
    'default_argument_type' => 'current_user',
    'default_argument' => '',
    'validate_type' => 'user',
    'validate_fail' => 'not found',
    'id' => 'completer_uid',
    'table' => 'cc_transactions',
    'field' => 'completer_uid',
    'validate_user_argument_type' => 'uid',
    'validate_user_roles' => array(
      '2' => 0,
      '3' => 0,
    ),
    'relationship' => 'none',
    'default_options_div_prefix' => '',
    'default_argument_user' => 0,
    'default_argument_fixed' => '',
    'default_argument_php' => '',
    'validate_argument_node_type' => array(
      'offer' => 0,
      'want' => 0,
      'requack' => 0,
      'transaction' => 0,
      'page' => 0,
      'story' => 0,
    ),
    'validate_argument_node_access' => 0,
    'validate_argument_nid_type' => 'nid',
    'validate_argument_vocabulary' => array(
      '2' => 0,
    ),
    'validate_argument_type' => 'tid',
    'validate_argument_transform' => 0,
    'validate_user_restrict_roles' => 0,
    'validate_argument_php' => '',
    'override' => array(
      'button' => 'Use default',
    ),
  ),
));
$handler->override_option('block_description', '');
$handler->override_option('block_caching', -1);
   
    
  return $view;
}

/** 
 / Implementation of hook_views_query_alter
  * reuses the statement argument so it shows transction fromand to a certain user
*/
function transactions_views_query_alter($view, &$query) {
  if ($view->name == 'statement') {
    $query->where[0]['clauses'][] = "cc_transactions.payee_uid = ".$query->where[0]['args'][0];
    $query->where[0]['type']= 'OR';
  }
}

/** 
 * Implementation of hook_views_pre_render
*/
function transactions_views_pre_render(&$view) { 
  if ($view->name == 'statement') {
    foreach ($view->result as $key=>$transaction){
      if ($transaction->cc_transactions_payer_uid == $view->payer_uid->argument) {
        $running_balance -= $transaction->cc_transactions_quantity;
        $view->result[$key]->expenditure = $transaction->cc_transactions_quantity;
        $view->result[$key]->income = NULL;
      } else {
        $running_balance += $transaction->cc_transactions_quantity;
        //we are repurposing the field payer_uid to mean other_person
        $view->result[$key]->cc_transactions_payer_uid = $transaction->cc_transactions_payer_uid;
        $view->result[$key]->income = $transaction->cc_transactions_quantity;
        $view->result[$key]->expenditure = NULL;
      }
    $view->result[$key]->running_balance = $running_balance;
    }
  }
}
//this seems to be expected if pre_render is present
function transactions_views_pre_view(){}
